{{
  *********************************************************
  *     Font_border_generator_demo  Version 1.0 - test    * 
  *             (C) 2007 Gennady Palitsky
  *          Jefferson Audio Video Systems                *
  *********************************************************

  Test program for generating a border around characters for a propellers buil-in font.
  Intended to work with tv text, 13 lines x 30 columns, interlaced

  The main idea:
  To generate a border around the font character, the built-in 16-bit 2-color characters are converted into
  32-bit 4-color (effectively 3-color) caracters, where separate color is designated for a border.

  Conversion is done on a row by row basis.
  For this purpose two row buffers (0 and 1) are arranged. Each buffer contains 32x16 longs.
  During the blanking interval the modified (not done yet) tv-driver informs the converter cog
  (by means of an unused pin flag) to convert the first row of a screen buffer.
  Since we use interlaced signal, only odd (for a first field) or even (second field) lines of original built-in characters
  are converted.
  Conversion should be done before tv-driver will start first row display.

  At a time of starting a row display tv-driver informs converter to start conversion of a second row into the row buffer 1.
  Instead of displaying characters from a built-in font, modified (not done) tv-driver starts displaying characters from
  a 4-color row buffer. By the time when row display is over, conversion for the next row (in row buffer 1 should be over).

  This way converter prepares, and driver displays alternating row buffers - when buffer 0 is displayed, buffer 1 is prepared,
  when buffer 1 is displayed, the the buffer 0 and so on, until all rows are displayed.

  In a second field everything is the same, but converter used even lines if a built-in character for a conversion.

  At this point I only implemented a single row buffer generation for the first field, just to test an idea.
  Implementing second buffer should not create a problem.

  Also tv-driver should be modified to display characters from a row buffer instaed of a screen buffer,
  and to generate signals for conversion start.
  Not done yet.

  Timing/optimization.
  At this moment the row conversion takes ~980 uS.
  Since display of arow takes 64uS x 16 lines, it should be fine.
  Nevertheless, just to be safe, it would be great to further optimize a conversion routine for speed.
  Any ideas are welcome.

  Acknowlegements:
  Parallax library drivers are used with or without modification.
  The border generation routine is based on idea of Phil Pilgrim to test each pixel against 8 neighbouring pixels.

}}   

      

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  Delay1 = 8000
  Delay2 = 400000

VAR

  long  stack[30]
  byte flag  

OBJ

  text : "tv_text_bg"
  serial: "Serial_transmit"
  

PUB main | ptr, addr, data

  'start tv_text 
  text.start(12)

  text.str(string("ABCD_abcd_0123_EFGH_efgh_4567_jk"))  'just a test string

  'start font converter in aseparate cog
  text.start_conv

  'start uart cog for debugging 
  serial.start(31, 30, 0, 57600)

  flag := 0

  'start a cog to generate test start signal for a converter
  cognew(blink(Delay1,Delay2),@stack[0])                

  ptr := text.get_rowptr        'get a pointer to a row_buffer in tv_text_bd 

  'debugging routine - prints developed row buffer to a serial port 
  repeat
    if (flag > 0)               'check if buffer development is over - flag is set in blink cog
    addr := ptr
    repeat 30                   '30 columns in a line
      repeat 16                 'row buffer is 16 lines x 30 columns
        data := long[addr]      'read data from a row buffer
        serial.print_charline(data)         'print one line in a row buffer in quaternary, reversed order (character line is reversed)            
        addr += 4                           'prepare to read next line in a row buffer
       serial.str(string(10, 13))           'cr
      serial.str(string(10, 13))            'cr after the whole character (16) lines is printed
    serial.str(string(10, 13))              'more crs after the whole buffer is printed
    serial.str(string(10, 13))
    flag := 0                               'inform bink cog that printing is over


PUB blink(del1,del2) | temp
{{
  Just generates signals to start conversion and waits till buffer printing is over
}}

  'P[2] - flag pin. Direction is set hete to out
  'font_convert cog is testing this pin and starts converting when it's high
  'effectively this is also a test for using unused pins as communication flags between cogs - it works! 
  outa[2]~                  'clear outputs
  dira[2]~~                 'set to outputs

  'just a dumb test loop
  repeat
    outa[2]~~                   'set P[2] - commands converter cog to start conversion
    waitcnt(del1 + cnt)         'delay should be less than conversion time
    outa[2]~                    'clear flag
    waitcnt(del2 + cnt)         '5 ms - definitely enough to finish conversion
    flag := 1                   'inform main to start printing buffer
    repeat                      'wait till printing is over 
      temp += 1                 'i didn't know what to insert for empty statement, so here we are
    while (flag > 0)  
    
        
    
            